#include "util.h"

#ifndef lint
char copyright[] =
"CALLBOOK - Version "VERSION"\nCopyright (c) 1995 Brian A. Lantz. All rights reserved.\n";
#endif /* not lint */


int outputmode;
char emptystring[1];


const char *classstr (int which);
char *timestr (time_t thetime);
void strupr (char *str);
void lookup (char *callsign, GDBM_FILE db);


const char *
classstr (which)
int which;
{
	switch (which)	{
		case TYPE_OP_NOVICE:		return ("Novice");
		case TYPE_OP_TECHNICIAN:	return ("Technician");
		case TYPE_OP_TECHNICIANPLUS:	return ("Technician Plus");
		case TYPE_OP_GENERAL:		return ("General");
		case TYPE_OP_ADVANCED:		return ("Advanced");
		case TYPE_OP_EXTRA:		return ("Extra");
		case TYPE_OP_UNKNOWN:
		default:			return ("Unknown!");
	}
}


char *
timestr (thetime)
time_t thetime;
{
static char time_str[80];
struct tm *t;

	if (!thetime)
		return (emptystring);
	t = localtime (&thetime);
	if (!outputmode)	{
#if 1
                (void) strftime (time_str, 80, "%B %d,%Y", t);
#else
		(void) strftime (time_str, 80, "%B %d,%Y  %I:%M:%S ", t);
		if (t->tm_hour > 11)
			strcat (time_str, "PM");
		else
			strcat (time_str, "AM");
#endif
	} else
		(void) strftime (time_str, 80, "%b-%d-%Y", t);
	return (time_str);
}


void
strupr (str)
char *str;
{
	while (*str)	{
		*str = toupper (*str);
		str++;
	}
}


void
lookup (callsign, db)
char *callsign;
GDBM_FILE db;
{
datum key_data, return_data;
struct appl_dat ap;
char buffer[60];

		strupr (callsign);
		key_data.dptr = callsign;
		key_data.dsize = strlen(callsign) + 1;
		return_data = gdbm_fetch (db, key_data);
		if (return_data.dptr != NULL)	{
			(void) unsqueeze (&ap, return_data.dptr);
			free (return_data.dptr);
			putchar ('\n');
			if (!outputmode)	{
				printf ("                     Callsign: %s\n", callsign);
				printf ("                         Name: %.40s\n", ap.name_licensee);
				printf ("                      Address: %.35s\n", ap.text_street);
				printf ("                     Address2: %.20s\n", ap.text_pobox);
				printf ("                         City: %.20s\n", ap.city);
				printf ("                        State: %.2s\n", ap.state);
				printf ("                      Zipcode: %.9s\n", ap.zipcode);
				printf ("                    Birthdate: %s\n", timestr(ap.date_of_birth));
				printf ("                  Date issued: %s\n", timestr(ap.date_issue));
				printf ("                 Date expires: %s\n", timestr(ap.date_expire));
				printf ("                  Last change: %s\n", timestr(ap.date_last_change));
				printf ("                License class: %s\n", classstr(ap.type_op_class));
				if (ap.callsign_prev[0])	{
					printf ("                Previous call: %s\n", ap.callsign_prev);
					printf ("               Prev lic class: %s\n", classstr(ap.type_prev_op_class));
				}
			} else	{
				strcpy (buffer, ap.text_street);
				if (ap.text_street[0] && ap.text_pobox[0])
					strcat (buffer, ", ");
				strcat (buffer, ap.text_pobox);
				printf ("%-18s%-42.40sISSUED  %s\n",
					callsign, ap.name_licensee, timestr(ap.date_issue));
				printf ("%-18s%-42.35sEXPIRES %s\n",
					classstr(ap.type_op_class),buffer,timestr(ap.date_expire));
				strcpy (buffer, timestr(ap.date_of_birth));
				printf ("DOB %-14s%-22.20s%-4.2s%-16.9sUPDATED %s\n",
					buffer,ap.city,ap.state,ap.zipcode,timestr(ap.date_last_change));
				if (ap.callsign_prev[0])
					printf ("%-18s(Previous call: %s, %s Class)\n",
						"", ap.callsign_prev, classstr(ap.type_prev_op_class));
			}
		} else
			printf ("No such callsign found\n");
}


int
main (int argc,char *argv[])
{
GDBM_FILE db;
char callsign[2048];
int done = 0, k;
char *sp;

	emptystring[0] = 0;
	sp = strrchr(argv[0],'/');
	if (sp)
		sp++;
	else
		sp = argv[0];
	outputmode = strcasecmp (sp, "callbook_www");

	db = gdbm_open (_PATH_DATABASE, 0, GDBM_READER, 00664, NULL);
	if (db == NULL)	{
		printf ("Error setting up gdbm file: %s\n", gdbm_strerror(gdbm_errno));
		return 0;
	}

	/* if called non-interactively */
	if (argc > 1)	{
		for (k = 1; k < argc; k++)
			lookup (argv[k], db);
	} else	{
		/* interactive usage */
		printf ("\n                          Tampa CallBook Server (CBS)\n");
		printf ("                                 Version "VERSION"\n\n");
		printf ("                      U.S. Amateur Radio Callsign Lookup\n");
		printf ("                    *** Individual's only at this time ***\n");
		printf ("               *** no alien, club, races, or military calls ***\n\n");

		printf ("The CallBook Server database is updated DAILY and taken directly from the FCC\n");
		printf ("database. This is made available to assist new hams waiting for their first\n");
		printf ("licence and hams waiting for their upgrades.  Under the new FCC rules, as soon\n");
		printf ("as you know your call, you can begin using it.\n\n");

		printf ("If you are running TNOS (or another xNOS) that has callserver support compiled\n");
		printf ("in, feel free to use this as your callserver host.\n\n");

		printf ("Any questions or bug reports regarding CBS should go to brian@lantz.com.\n\n");

		done = 0;
		do	{
			printf ("\nEnter callsign (blank line to exit): ");
			fflush (stdout);
			gets (callsign);
			if (callsign[strlen(callsign) - 1] == '\r')
				callsign[strlen(callsign) - 1] = 0;
			if (callsign[strlen(callsign) - 1] == '\n')
				callsign[strlen(callsign) - 1] = 0;
			if (!*callsign)	{
				done = 1;
				continue;
			}
			lookup (callsign, db);
		} while (!done);
	}
	gdbm_close (db);
	return 0;
}

